@@ -102,11 +102,12 @@ use crossterm::{
   QueueableCommand,
 };
 use std::{
+  fmt,
   io::{stdin, stdout, Stdout, Write},
   str::FromStr,
 };
 /// Anything that can be listed in `Menu`.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub enum Item {
   /// A menu item to execute an action. Exits `Menu`.
   Action {
@@ -183,6 +184,11 @@ pub enum Item {
     exp: Option<String>,
   },
 }
+impl fmt::Display for Item {
+  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
+    write!(f, "{:?}", self)
+  }
+}
 /// Starting point for creating a menu instance.
 pub struct Menu {
   /// `Menu` name is displayed at the top.
@@ -203,6 +209,8 @@ pub struct Selection {
   pub path: Vec<String>,
   /// Input by user, if it exists.
   pub value: Option<Value>,
+  /// Length of value.
+  pub len: Option<usize>,
   /// Number of attempts of input.
   pub attempt: Option<i32>,
 }
@@ -223,10 +231,15 @@ impl Menu {
     let mut hover = 0 as usize;
     self.printer(&mut stdout_ins, &mut hover)
   }
+  fn rerun(&self, hover: usize) -> Result<Selection, String> {
+    let mut stdout_ins = stdout();
+    let mut hover = hover;
+    self.printer(&mut stdout_ins, &mut hover)
+  }
   fn printer(&self, stdout_ins: &mut Stdout, hover: &mut usize) -> Result<Selection, String> {
     self.print_top(&vec![self.name.to_string()]);
     self.print_items(hover);
-    self.print_bottom(false);
+    self.print_bottom(false, hover);
     self.matcher(stdout_ins, hover)
   }
   fn matcher(&self, stdout_ins: &mut Stdout, hover: &mut usize) -> Result<Selection, String> {
@@ -250,6 +263,11 @@ impl Menu {
     let mut hover = 0 as usize;
     self.printer_sub(path, &mut stdout_ins, &mut hover)
   }
+  fn rerun_sub(&self, path: &mut Vec<String>, hover: usize) -> Result<Selection, String> {
+    let mut stdout_ins = stdout();
+    let mut hover = hover;
+    self.printer_sub(path, &mut stdout_ins, &mut hover)
+  }
   fn printer_sub(
     &self,
     path: &mut Vec<String>,
@@ -258,7 +276,7 @@ impl Menu {
   ) -> Result<Selection, String> {
     self.print_top(path);
     self.print_items(hover);
-    self.print_bottom(true);
+    self.print_bottom(true, hover);
     self.matcher_sub(stdout_ins, path, hover)
   }
   fn matcher_sub(
@@ -298,8 +316,11 @@ impl Menu {
           self.print_hotkey(&i, hotkey);
           self.print_name_exp(&i, hover, true, &("+".to_owned() + name), exp);
         }
-        Item::Bool { name, hotkey, exp }
-        | Item::Char { name, hotkey, exp }
+        Item::Bool { name, hotkey, exp } => {
+          self.print_hotkey(&i, hotkey);
+          self.print_name_exp(&i, hover, true, &("+".to_owned() + name + "="), exp);
+        }
+        Item::Char { name, hotkey, exp }
         | Item::String { name, hotkey, exp }
         | Item::F64 { name, hotkey, exp }
         | Item::I64 { name, hotkey, exp }
@@ -310,25 +331,45 @@ impl Menu {
       }
     }
   }
-  fn print_bottom(&self, is_sub: bool) {
+  fn print_bottom(&self, is_sub: bool, hover: &mut usize) {
     print!(
-      "{}{}{}{}{}{}{}{}{}{}",
+      "{}{}{}{}{}{}{}",
       "(".dark_grey(),
       "Up".yellow(),
       ")".dark_grey(),
-      ", (".dark_grey(),
+      "(".dark_grey(),
       "Down".yellow(),
-      ")".dark_grey(),
-      ", (".dark_grey(),
-      "Enter".yellow(),
       ") ".dark_grey(),
-      "Select",
+      "Move",
     );
-    if is_sub {
+    if let Item::SubMenu { .. } | Item::Bool { .. } = self.items[*hover] {
+      print!(
+        "{}{}{}{}{}{}{}",
+        ", (".dark_grey(),
+        "Enter".yellow(),
+        ")".dark_grey(),
+        "(".dark_grey(),
+        "Right".yellow(),
+        ") ".dark_grey(),
+        "Select",
+      );
+    } else {
       print!(
         "{}{}{}{}",
         ", (".dark_grey(),
+        "Enter".yellow(),
+        ") ".dark_grey(),
+        "Select",
+      );
+    }
+    if is_sub {
+      print!(
+        "{}{}{}{}{}{}{}",
+        ", (".dark_grey(),
         "Backspace".yellow(),
+        ")".dark_grey(),
+        "(".dark_grey(),
+        "Left".yellow(),
         ") ".dark_grey(),
         "Back",
       );
@@ -369,6 +410,8 @@ impl Menu {
     match keycode {
       KeyCode::Up => Some(String::from("Up")),
       KeyCode::Down => Some(String::from("Down")),
+      KeyCode::Left => Some(String::from("Left")),
+      KeyCode::Right => Some(String::from("Right")),
       KeyCode::Enter => Some(String::from("Enter")),
       KeyCode::Esc => Some(String::from("Exit")),
       KeyCode::Backspace => Some(String::from("Back")),
@@ -386,7 +429,7 @@ impl Menu {
   ) -> Result<Selection, String> {
     if *key == None {
       return Err("No Selection".to_string());
-    } else if is_sub && *key == Some("Back".to_string()) {
+    } else if is_sub && (*key == Some("Back".to_string()) || *key == Some("Left".to_string())) {
       self.clear_menu(stdout_ins);
       return Err("Back".to_string());
     } else if *key == Some("Exit".to_string()) {
@@ -398,23 +441,33 @@ impl Menu {
     } else if *key == Some("Up".to_string()) {
       if *hover > 0 {
         *hover -= 1;
-        self.clear_menu(stdout_ins);
-        if path.len() == 1 {
-          return self.printer(stdout_ins, hover);
-        } else {
-          return self.printer_sub(path, stdout_ins, hover);
-        }
+      } else {
+        *hover = self.items.len() - 1;
       }
+      self.clear_menu(stdout_ins);
+      self.print_top(path);
+      self.print_items(hover);
+      if path.len() == 1 {
+        self.print_bottom(false, hover);
+      } else {
+        self.print_bottom(true, hover);
+      }
+      return Err("No Selection".to_string());
     } else if *key == Some("Down".to_string()) {
       if (*hover + 1) < self.items.len() {
         *hover += 1;
-        self.clear_menu(stdout_ins);
-        if path.len() == 1 {
-          return self.printer(stdout_ins, hover);
-        } else {
-          return self.printer_sub(path, stdout_ins, hover);
-        }
+      } else {
+        *hover = 0;
       }
+      self.clear_menu(stdout_ins);
+      self.print_top(path);
+      self.print_items(hover);
+      if path.len() == 1 {
+        self.print_bottom(false, hover);
+      } else {
+        self.print_bottom(true, hover);
+      }
+      return Err("No Selection".to_string());
     }
     for (i, item) in self.items.iter().enumerate() {
       match item {
@@ -430,6 +483,7 @@ impl Menu {
               name: name.to_string(),
               path: path.to_vec(),
               value: None,
+              len: None,
               attempt: None,
             });
           } else {
@@ -444,7 +498,8 @@ impl Menu {
         } => {
           if (*key == hotkey.map(|f| f.to_string()))
             || (*key == Some(i.to_string()))
-            || (*key == Some("Enter".to_string()) && i == *hover)
+            || ((*key == Some("Enter".to_string()) || *key == Some("Right".to_string()))
+              && i == *hover)
           {
             self.clear_menu(stdout_ins);
             path.push(name.to_string());
@@ -460,9 +515,9 @@ impl Menu {
               Err(err) if &err == "Back" => {
                 path.pop();
                 if path.len() == 1 {
-                  return self.run();
+                  return self.rerun(*hover);
                 } else {
-                  return self.run_sub(path);
+                  return self.rerun_sub(path, *hover);
                 }
               }
               Err(err) => return Err(err),
@@ -474,7 +529,8 @@ impl Menu {
         Item::Bool { name, hotkey, exp } => {
           if (*key == hotkey.map(|f| f.to_string()))
             || (*key == Some(i.to_string()))
-            || (*key == Some("Enter".to_string()) && i == *hover)
+            || ((*key == Some("Enter".to_string()) || *key == Some("Right".to_string()))
+              && i == *hover)
           {
             self.clear_menu(stdout_ins);
             path.push(name.to_string());
@@ -498,18 +554,16 @@ impl Menu {
             let sub_result = sub_menu.run_sub(path);
             match sub_result {
               Ok(mut ok) => {
-                return {
-                  let last = ok.path.pop().expect("item bool path pop");
-                  ok.value = Some(Value::Bool(last.parse().expect("item bool value parse")));
-                  Ok(ok)
-                }
+                let last = ok.path.pop().expect("item bool path pop");
+                ok.value = Some(Value::Bool(last.parse().expect("item bool value parse")));
+                return Ok(ok);
               }
               Err(err) if &err == "Back" => {
                 path.pop();
                 if path.len() == 1 {
-                  return self.run();
+                  return self.rerun(*hover);
                 } else {
-                  return self.run_sub(path);
+                  return self.rerun_sub(path, *hover);
                 }
               }
               Err(err) => return Err(err),
@@ -532,55 +586,60 @@ impl Menu {
             path.push(name.to_string());
             // (done): print
             self.print_top(path);
-            self.print_name(name, exp);
+            self.print_name(item, name, exp);
             // (done): selection
             let mut attempt = 1;
             let input = self.read_line_string();
             let selection = match item {
               Item::Char { .. } => {
-                let value: char = self.match_input(input, &mut attempt);
+                let value: char = self.match_input(item, input, &mut attempt);
                 Selection {
                   name: name.to_string(),
                   path: path.to_vec(),
                   value: Some(Value::Char(value)),
+                  len: Some(1),
                   attempt: Some(attempt),
                 }
               }
               Item::F64 { .. } => {
-                let value: f64 = self.match_input(input, &mut attempt);
+                let value: f64 = self.match_input(item, input, &mut attempt);
                 Selection {
                   name: name.to_string(),
                   path: path.to_vec(),
                   value: Some(Value::F64(value)),
+                  len: Some(value.to_string().len()),
                   attempt: Some(attempt),
                 }
               }
               Item::I64 { .. } => {
-                let value: i64 = self.match_input(input, &mut attempt);
+                let value: i64 = self.match_input(item, input, &mut attempt);
                 Selection {
                   name: name.to_string(),
                   path: path.to_vec(),
                   value: Some(Value::I64(value)),
+                  len: Some(value.to_string().len()),
                   attempt: Some(attempt),
                 }
               }
               Item::U64 { .. } => {
-                let value: u64 = self.match_input(input, &mut attempt);
+                let value: u64 = self.match_input(item, input, &mut attempt);
                 Selection {
                   name: name.to_string(),
                   path: path.to_vec(),
                   value: Some(Value::U64(value)),
+                  len: Some(value.to_string().len()),
                   attempt: Some(attempt),
                 }
               }
               _ => Selection {
                 name: name.to_string(),
                 path: path.to_vec(),
-                value: Some(Value::String(input)),
+                value: Some(Value::String(String::from(&input))),
+                len: Some(input.len()),
                 attempt: Some(attempt),
               },
             };
-            self.clear_lines(stdout_ins, (3 + attempt) as u16);
+            self.clear_lines(stdout_ins, (2 + (attempt * 2)) as u16);
             stdout_ins.flush().unwrap();
             return Ok(selection);
           } else {
@@ -614,7 +673,7 @@ impl Menu {
       None => print!("   "),
     }
   }
-  fn print_name(&self, name: &String, item_exp: &Option<String>) {
+  fn print_name(&self, item: &Item, name: &String, item_exp: &Option<String>) {
     if let Some(item_exp) = item_exp {
       println!(
         "       {} {}",
@@ -627,7 +686,18 @@ impl Menu {
         String::from(name.to_owned() + "=").cyan().bold()
       );
     }
-    println!("{}", "Enter value:".dark_grey());
+    self.print_input_bottom(item);
+  }
+  fn print_input_bottom(&self, item: &Item) {
+    // (done): slice
+    let string = item.to_string();
+    let first = string
+      .find("{")
+      .expect("struct name find first parenthesis");
+    let slice = string[0..first - 1].to_string();
+    // (done): print
+    print!("{}{}", "Enter a value. Type: ".dark_grey(), slice.blue());
+    println!();
   }
   fn print_name_exp(
     &self,
@@ -658,13 +728,15 @@ impl Menu {
     stdin().read_line(&mut input).expect("read line");
     input.trim().to_string()
   }
-  fn match_input<T: FromStr>(&self, input: String, attempt: &mut i32) -> T {
+  fn match_input<T: FromStr>(&self, item: &Item, input: String, attempt: &mut i32) -> T {
     match input.parse() {
       Ok(ok) => ok,
       Err(_) => {
         *attempt += 1;
+        print!("{}", "Invalid entry: ".dark_red(),);
+        self.print_input_bottom(item);
         let input = self.read_line_string();
-        self.match_input(input, attempt)
+        self.match_input(item, input, attempt)
       }
     }
   }
